@@ -21,15 +21,18 @@ def main():
     options = parser.parse_args()
     common.options = options
     pkgs = common.read_pkg_args(options.appid, True)
-    allapps = metadata.read_metadata(pkgs, check_vcs=True)
+    allapps = metadata.read_metadata(pkgs)
     apps = common.read_app_args(options.appid, allapps, True)
+    common.read_config(options)
     srclib_dir = os.path.join('build', 'srclib')
     os.makedirs(srclib_dir, exist_ok=True)
     srclibpaths = []
     for appid, app in apps.items():
+        vcs, _ignored = common.setup_vcs(app)
+        vcs.gotorevision('HEAD', refresh=False)
         for build in app.get('Builds', []):
             for lib in build.srclibs:
-                srclibpaths.append(common.getsrclib(lib, srclib_dir, build=build))
+                srclibpaths.append(common.getsrclib(lib, srclib_dir, prepare=False, build=build))
     print('Set up srclibs:')
     pprint.pprint(srclibpaths)
 
