@@ -40,13 +40,12 @@ COMMANDS = OrderedDict([
     ("deploy", _("Interact with the repo HTTP server")),
     ("verify", _("Verify the integrity of downloaded packages")),
     ("checkupdates", _("Check for updates to applications")),
-    ("import", _("Add a new application from its source code")),
+    ("import", _("Extract application metadata from a source repository")),
     ("install", _("Install built packages on devices")),
     ("readmeta", _("Read all the metadata files and exit")),
     ("rewritemeta", _("Rewrite all the metadata files")),
     ("lint", _("Warn about possible metadata errors")),
     ("scanner", _("Scan the source code of a package")),
-    ("stats", _("Update the stats of the repo")),
     ("signindex", _("Sign indexes created using update --nosign")),
     ("btlog", _("Update the binary transparency log for a URL")),
     ("signatures", _("Extract signatures from APKs")),
@@ -159,7 +158,6 @@ def main():
                         output = 'git commit ' + subprocess.check_output(['git', 'rev-parse', 'HEAD'],
                                                                          universal_newlines=True)
                 elif os.path.exists('setup.py'):
-                    import re
                     m = re.search(r'''.*[\s,\(]+version\s*=\s*["']([0-9a-z.]+)["'].*''',
                                   open('setup.py').read(), flags=re.MULTILINE)
                     if m:
@@ -197,6 +195,8 @@ def main():
 
     del sys.argv[1]
     if command in COMMANDS.keys():
+        # import is named import_subcommand internally b/c import is reserved by Python
+        command = 'import_subcommand' if command == 'import' else command
         mod = __import__('fdroidserver.' + command, None, None, [command])
     else:
         mod = __import__(available_plugins[command]['name'], None, None, [command])
