@@ -25,6 +25,7 @@ import re
 import subprocess
 import time
 import urllib
+import yaml
 from argparse import ArgumentParser
 import logging
 import shutil
@@ -46,6 +47,34 @@ USER_S3CFG = 's3cfg'
 REMOTE_HOSTNAME_REGEX = re.compile(r'\W*\w+\W+(\w+).*')
 
 
+def _get_index_excludes(repo_section):
+    """Return the list of files to be synced last, since they finalize the deploy.
+
+    The process of pushing all the new packages to the various
+    services can take a while.  So the index files should be updated
+    last.  That ensures that the package files are available when the
+    client learns about them from the new index files.
+
+    """
+    indexes = [
+        os.path.join(repo_section, 'entry.jar'),
+        os.path.join(repo_section, 'entry.json'),
+        os.path.join(repo_section, 'entry.json.asc'),
+        os.path.join(repo_section, 'index-v1.jar'),
+        os.path.join(repo_section, 'index-v1.json'),
+        os.path.join(repo_section, 'index-v1.json.asc'),
+        os.path.join(repo_section, 'index-v2.json'),
+        os.path.join(repo_section, 'index-v2.json.asc'),
+        os.path.join(repo_section, 'index.jar'),
+        os.path.join(repo_section, 'index.xml'),
+    ]
+    index_excludes = []
+    for f in indexes:
+        index_excludes.append('--exclude')
+        index_excludes.append(f)
+    return index_excludes
+
+
 def update_awsbucket(repo_section):
     """Upload the contents of the directory `repo_section` (including subdirectories) to the AWS S3 "bucket".
 
@@ -104,33 +133,23 @@ def update_awsbucket_s3cmd(repo_section):
         s3cmd_sync += ['--verbose']
     if options.quiet:
         s3cmd_sync += ['--quiet']
-    indexxml = os.path.join(repo_section, 'index.xml')
-    indexjar = os.path.join(repo_section, 'index.jar')
-    indexv1jar = os.path.join(repo_section, 'index-v1.jar')
-    indexv1json = os.path.join(repo_section, 'index-v1.json')
-    indexv1jsonasc = os.path.join(repo_section, 'index-v1.json.asc')
 
     s3url = s3bucketurl + '/fdroid/'
     logging.debug('s3cmd sync new files in ' + repo_section + ' to ' + s3url)
     logging.debug(_('Running first pass with MD5 checking disabled'))
-    if subprocess.call(s3cmd_sync
-                       + ['--no-check-md5', '--skip-existing',
-                          '--exclude', indexxml,
-                          '--exclude', indexjar,
-                          '--exclude', indexv1jar,
-                          '--exclude', indexv1json,
-                          '--exclude', indexv1jsonasc,
-                          repo_section, s3url]) != 0:
+    excludes = _get_index_excludes(repo_section)
+    returncode = subprocess.call(
+        s3cmd_sync
+        + excludes
+        + ['--no-check-md5', '--skip-existing', repo_section, s3url]
+    )
+    if returncode != 0:
         raise FDroidException()
     logging.debug('s3cmd sync all files in ' + repo_section + ' to ' + s3url)
-    if subprocess.call(s3cmd_sync
-                       + ['--no-check-md5',
-                          '--exclude', indexxml,
-                          '--exclude', indexjar,
-                          '--exclude', indexv1jar,
-                          '--exclude', indexv1json,
-                          '--exclude', indexv1jsonasc,
-                          repo_section, s3url]) != 0:
+    returncode = subprocess.call(
+        s3cmd_sync + excludes + ['--no-check-md5', repo_section, s3url]
+    )
+    if returncode != 0:
         raise FDroidException()
 
     logging.debug(_('s3cmd sync indexes {path} to {url} and delete')
@@ -241,8 +260,22 @@ def update_awsbucket_libcloud(repo_section):
 
 
 def update_serverwebroot(serverwebroot, repo_section):
-    # use a checksum comparison for accurate comparisons on different
-    # filesystems, for example, FAT has a low resolution timestamp
+    """Deploy the index files to the serverwebroot using rsync.
+
+    Upload the first time without the index files and delay the
+    deletion as much as possible.  That keeps the repo functional
+    while this update is running.  Then once it is complete, rerun the
+    command again to upload the index files.  Always using the same
+    target with rsync allows for very strict settings on the receiving
+    server, you can literally specify the one rsync command that is
+    allowed to run in ~/.ssh/authorized_keys.  (serverwebroot is
+    guaranteed to have a trailing slash in common.py)
+
+    It is possible to optionally use a checksum comparison for
+    accurate comparisons on different filesystems, for example, FAT
+    has a low resolution timestamp
+
+    """
     rsyncargs = ['rsync', '--archive', '--delete-after', '--safe-links']
     if not options.no_checksum:
         rsyncargs.append('--checksum')
@@ -254,26 +287,9 @@ def update_serverwebroot(serverwebroot, repo_section):
         rsyncargs += ['-e', 'ssh -oBatchMode=yes -oIdentitiesOnly=yes -i ' + options.identity_file]
     elif 'identity_file' in config:
         rsyncargs += ['-e', 'ssh -oBatchMode=yes -oIdentitiesOnly=yes -i ' + config['identity_file']]
-    indexxml = os.path.join(repo_section, 'index.xml')
-    indexjar = os.path.join(repo_section, 'index.jar')
-    indexv1jar = os.path.join(repo_section, 'index-v1.jar')
-    indexv1json = os.path.join(repo_section, 'index-v1.json')
-    indexv1jsonasc = os.path.join(repo_section, 'index-v1.json.asc')
-    # Upload the first time without the index files and delay the deletion as
-    # much as possible, that keeps the repo functional while this update is
-    # running.  Then once it is complete, rerun the command again to upload
-    # the index files.  Always using the same target with rsync allows for
-    # very strict settings on the receiving server, you can literally specify
-    # the one rsync command that is allowed to run in ~/.ssh/authorized_keys.
-    # (serverwebroot is guaranteed to have a trailing slash in common.py)
     logging.info('rsyncing ' + repo_section + ' to ' + serverwebroot)
-    if subprocess.call(rsyncargs
-                       + ['--exclude', indexxml,
-                          '--exclude', indexjar,
-                          '--exclude', indexv1jar,
-                          '--exclude', indexv1json,
-                          '--exclude', indexv1jsonasc,
-                          repo_section, serverwebroot]) != 0:
+    excludes = _get_index_excludes(repo_section)
+    if subprocess.call(rsyncargs + excludes + [repo_section, serverwebroot]) != 0:
         raise FDroidException()
     if subprocess.call(rsyncargs + [repo_section, serverwebroot]) != 0:
         raise FDroidException()
@@ -361,7 +377,8 @@ def update_servergitmirrors(servergitmirrors, repo_section):
     if repo_section == 'repo':
         git_mirror_path = 'git-mirror'
         dotgit = os.path.join(git_mirror_path, '.git')
-        git_repodir = os.path.join(git_mirror_path, 'fdroid', repo_section)
+        git_fdroiddir = os.path.join(git_mirror_path, 'fdroid')
+        git_repodir = os.path.join(git_fdroiddir, repo_section)
         if not os.path.isdir(git_repodir):
             os.makedirs(git_repodir)
         # github/gitlab use bare git repos, so only count the .git folder
@@ -413,16 +430,28 @@ def update_servergitmirrors(servergitmirrors, repo_section):
         repo.index.commit("fdroidserver git-mirror")
 
         if options.verbose:
-            bar = progress.Bar()
+            progressbar = progress.Bar()
 
             class MyProgressPrinter(git.RemoteProgress):
                 def update(self, op_code, current, maximum=None, message=None):
                     if isinstance(maximum, float):
-                        bar.show(current, maximum)
+                        progressbar.show(current, maximum)
             progress = MyProgressPrinter()
         else:
             progress = None
 
+        # only deploy to GitLab Artifacts if too big for GitLab Pages
+        if common.get_dir_size(git_fdroiddir) <= common.GITLAB_COM_PAGES_MAX_SIZE:
+            gitlab_ci_job_name = 'pages'
+        else:
+            gitlab_ci_job_name = 'GitLab Artifacts'
+            logging.warning(
+                _(
+                    'Skipping GitLab Pages mirror because the repo is too large (>%.2fGB)!'
+                )
+                % (common.GITLAB_COM_PAGES_MAX_SIZE / 1000000000)
+            )
+
         # push for every remote. This will overwrite the git history
         for remote in repo.remotes:
             if remote.name not in enabled_remotes:
@@ -430,16 +459,22 @@ def update_servergitmirrors(servergitmirrors, repo_section):
                 continue
             if remote.name == 'gitlab':
                 logging.debug('Writing .gitlab-ci.yml to deploy to GitLab Pages')
-                with open(os.path.join(git_mirror_path, ".gitlab-ci.yml"), "wt") as out_file:
-                    out_file.write("""pages:
-  script:
-   - mkdir .public
-   - cp -r * .public/
-   - mv .public public
-  artifacts:
-    paths:
-    - public
-""")
+                with open(os.path.join(git_mirror_path, ".gitlab-ci.yml"), "wt") as fp:
+                    yaml.dump(
+                        {
+                            gitlab_ci_job_name: {
+                                'script': [
+                                    'mkdir .public',
+                                    'cp -r * .public/',
+                                    'mv .public public',
+                                ],
+                                'artifacts': {'paths': ['public']},
+                                'variables': {'GIT_DEPTH': 1},
+                            }
+                        },
+                        fp,
+                        default_flow_style=False,
+                    )
 
                 repo.git.add(all=True)
                 repo.index.commit("fdroidserver git-mirror: Deploy to GitLab Pages")
@@ -462,7 +497,7 @@ def update_servergitmirrors(servergitmirrors, repo_section):
                         logging.debug(remote.url + ': ' + pushinfo.summary)
 
         if progress:
-            bar.done()
+            progressbar.done()
 
 
 def upload_to_android_observatory(repo_section):
@@ -490,7 +525,7 @@ def upload_apk_to_android_observatory(path):
     apkfilename = os.path.basename(path)
     r = requests.post('https://androidobservatory.org/',
                       data={'q': update.sha256sum(path), 'searchby': 'hash'},
-                      headers=net.HEADERS)
+                      headers=net.HEADERS, timeout=300)
     if r.status_code == 200:
         # from now on XPath will be used to retrieve the message in the HTML
         # androidobservatory doesn't have a nice API to talk with
@@ -518,7 +553,7 @@ def upload_apk_to_android_observatory(path):
     r = requests.post('https://androidobservatory.org/upload',
                       files={'apk': (apkfilename, open(path, 'rb'))},
                       headers=net.HEADERS,
-                      allow_redirects=False)
+                      allow_redirects=False, timeout=300)
 
 
 def upload_to_virustotal(repo_section, virustotal_apikey):
@@ -570,7 +605,7 @@ def upload_apk_to_virustotal(virustotal_apikey, packageName, apkName, hash,
     needs_file_upload = False
     while True:
         r = requests.get('https://www.virustotal.com/vtapi/v2/file/report?'
-                         + urllib.parse.urlencode(data), headers=headers)
+                         + urllib.parse.urlencode(data), headers=headers, timeout=300)
         if r.status_code == 200:
             response = r.json()
             if response['response_code'] == 0:
@@ -604,7 +639,7 @@ def upload_apk_to_virustotal(virustotal_apikey, packageName, apkName, hash,
         elif size > 32000000:
             # VirusTotal API requires fetching a URL to upload bigger files
             r = requests.get('https://www.virustotal.com/vtapi/v2/file/scan/upload_url?'
-                             + urllib.parse.urlencode(data), headers=headers)
+                             + urllib.parse.urlencode(data), headers=headers, timeout=300)
             if r.status_code == 200:
                 upload_url = r.json().get('upload_url')
             elif r.status_code == 403:
@@ -622,7 +657,7 @@ def upload_apk_to_virustotal(virustotal_apikey, packageName, apkName, hash,
         files = {
             'file': (apkName, open(repofilename, 'rb'))
         }
-        r = requests.post(upload_url, data=data, headers=headers, files=files)
+        r = requests.post(upload_url, data=data, headers=headers, files=files, timeout=300)
         logging.debug(_('If this upload fails, try manually uploading to {url}')
                       .format(url=manual_url))
         r.raise_for_status()
