@@ -98,7 +98,7 @@ def main():
         # exist, prompt the user using platform-specific default
         # and if the user leaves it blank, ignore and move on.
         default_sdk_path = ''
-        if sys.platform == 'win32' or sys.platform == 'cygwin':
+        if sys.platform in ('win32', 'cygwin'):
             p = os.path.join(
                 os.getenv('USERPROFILE'), 'AppData', 'Local', 'Android', 'android-sdk'
             )
@@ -264,7 +264,7 @@ def main():
         c = dict(test_config)
         c['keystorepass'] = password
         c['keypass'] = password
-        c['repo_keyalias'] = socket.getfqdn()
+        c['repo_keyalias'] = repo_keyalias or socket.getfqdn()
         c['keydname'] = 'CN=' + c['repo_keyalias'] + ', OU=F-Droid'
         common.write_to_config(test_config, 'keystorepass', password)
         common.write_to_config(test_config, 'keypass', password)
